############################################################################
#
#  Licensed Materials - Property of IBM
#  5698-NVW
#
#  (C) Copyright IBM Corp. 2002.  All Rights Reserved.
#
#  US Government Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with
#  IBM Corp.
#
#   File Name: ..\misc\anm_create_hours.pl
#   JMP - Added all comments Aug, 2005
#
############################################################################

use Time::Local;

if (! @ARGV) {
    $log = 'anm_create_hours.log';
    if (! open(LOG, ">$log")) {
        print "Create Hours Error: Unable to open the specified log file for writing:\n";
        print "\n    \"$log\".\n\n";
        print "$!\n";
        exit(1);
    }
    print LOG "\Create Hours Error: a file name is required.\n\n";
    usage(1);
}

$log = $ARGV[0].'.log';
if (! open(LOG, ">$log")) {
    print "\nCreate Hours Error: Unable to open the specified log file for writing:\n";
    print "\n    \"$log\".\n\n";
    print "$!\n";
    exit(1);
    }
print LOG "Logfile $log opened.\n";

$hfile = $ARGV[0].'.preedit';
if (! open(HOSTS, "<$hfile")) {
    print LOG "\nCreate Hours Error: unable to open the specified hostname file for reading:\n";
    print LOG "\n    \"$hfile\".\n\n";
    print LOG "$!\n";
    close(LOG);
    exit(1);
}

 $out = $ARGV[0].'.postedit';
 if (! open(OUT, ">$out")) {
        print LOG "\nCreate Hours Error: Unable to open the specified output file for writing:\n";
        print LOG "\n    \"$out\".\n\n";
        print LOG "$!\n";
        close(LOG);
        exit(1);
    }

while ($inline = <HOSTS>) {
    @inlist = split_commas($inline);
#inlist[0] is the start_hour from anm.stage_temp_hours
#inlist[1] is the end_hour from anm.stage_temp_hours
#They are in the format yyyy-monmon-dd-hh.minmin.ss.000000
    print LOG ">>Processing Input: \"$inlist[0]\",\"$inlist[1]\" \n";
    @outline = ();

    @times = ();
    @times2 = ();
    
# The array times = "yyyy", "mm", "dd", "hhhh.minmin.ss.000000"
    @times = split(/-/, $inlist[1]);

# times[3] = "hhhh.minmin.ss.000000"
    @times2 = split(/\./, $times[3]);
# Therefore the array "times2" = "hhhh", "minmin", "ss", "000000"


# Temp time has always included $times[1] - 1; Because January = 0 and the date is currently standard
# For timegm(broken-down time), get back seconds in UTC
   $endtemptime = timegm($times2[2], $times2[1], $times2[0], $times[2], $times[1] - 1, $times[0]);

# The stuff being passed to endtemptime should be ss, minmin, hh, dd, monmon, yyy 
# where month starts with 0 and year is yyyy - 1900
# The response should be seconds in UTC since the epoch
    @end = gmtime($endtemptime);

    if ( $inlist[0] eq "1000-01-01-00.00.00.000000" ) {
	$eyear = $end[5] + 1900;
	$emon = $end[4];
	print OUT "\"$eyear-$emon-$end[3]-00\.00\.00\.000000\"\n";
    }
    else {

# ? Why was there no inlist[0] being split up?
	$inlist[0] =~ s/\s+$//;
	$inlist[1] =~ s/\s+$//;

	@times = ();
	@times2 = ();

	@times = split(/-/, $inlist[0]);

	@times2 = split(/\./, $times[3]);

# PJ23653 Changed "$times[1] - 1" to "$times[1] + 1"
# 148477 Changed "$times[1] + 1" back to "$times[1] - 1"

	$starttemptime = timegm($times2[2], $times2[1], $times2[0], $times[2], $times[1] - 1, $times[0]);

	@start = gmtime($starttemptime);
	
	if ( ($starttemptime != $endtemptime) ) {

	    while ( ( ($start[7] <= $end[7]) && ($start[5] = $end[5]) ) ||($start[5] < $end[5]) )  {
	    
		$ttime = timegm(0, 0, 0, $start[3], $start[4], $start[5]);
		@tempoutline3 = ();
		@tempoutline3 = gmtime($ttime + 86400);
		$tyear = $tempoutline3[5] + 1900;
		$tmon = $tempoutline3[4] + 1;
		$tday = $tempoutline3[3];
		$thour = $tempoutline3[2];
		$tmin = $tempoutline3[1];
		$tsec = $tempoutline3[0];
		print OUT "\"$tyear-$tmon-$tday-$thour\.";
		print OUT "00\.00\.000000\"\n";
		
		@start = @tempoutline3;
		
	    }
	    
	}

    }
}

print LOG "\nEnd of Input.\n";
print LOG "End of Processing.\n";

close(HOSTS);
close(OUT);
close(LOG);

exit(0);

sub usage {
    $rc = @_;
    print LOG <<USAGE;
anm_split_outages.pl <HOSTS_FILE>\n
	where HOSTS_FILE.preedit is a csv file with 'start_time','end_time'
USAGE
    close(LOG);
    exit($rc);
}

sub split_commas {
    my $line = shift;
    my @listitems = ();
    push(@listitems, $+) while $line =~ m{
    "([^\"\\]*(?:\\.[^\"\\]*)*)",?
      |  ([^,]+),?
      | ,
    }gx;
    push(@listitems, undef) if substr($line, -1,1) eq ',';
    return @listitems;
}
